/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 * 
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import java.awt.Graphics2D;
import java.awt.Image;
import java.util.ArrayList;
import javax.swing.JLabel;

/**
 *
 * @author Emerson Ribeiro de Mello
 */
public class Jogador extends Elemento {

    protected int cima; // código da tecla para mover para cima
    protected int baixo;// código da tecla para mover para baixo
    protected int direita;// código da tecla para mover para direita
    protected int esquerda;// código da tecla para mover para esquerda
    protected int atirar;// código da tecla para atirar
    // as 2 linhas abaixo são usadas so' para depuracao
    private JLabel coordX; // JLabel no Frame principal para representar a coordenada X
    private JLabel coordY;// JLabel no Frame principal para representar a coordenada Y
    protected Bomba bomba;// cria uma bomba
    private ArrayList<Bomba> qntDebombas; // vetor de bombas
    protected int bombasSimultaneas = 1; // valor de bombas que o jogador pode colocar simultaneamente
    protected int vida = 3; // vida iniciais dos jogadores
    protected int tempoAtingido = 0; // tempo em que o jogador fica sentindo a explosao da bomba
    protected boolean mostraJogador = true; // serve para fazer o jogador piscar quando for atingido por bombas.
    protected int tipoDeVeneno = 0; // para decidir se diminui chama, bomba ou inverte comandos movimentacao
    protected int tempoDeInversaoDeTecla = 0; // tempo que o jogador fica com as teclas invertidas
    protected boolean inverteuTeclas = false; // para saber se as teclas jah foram invertidas

    /**
     * 
     * @param pai referencia a area de Desenho
     * @param imagemNome - Nome da imagem que sera' usada para representar o jogador
     * @param bombaNome  - Nome da imagem quse sera' usada para representar a bomba deste jogador
     * @param mX Linha na matriz
     * @param mY Coluna na matriz
     * @param c codigo da tecla que sera' usado para mover para cima
     * @param b codigo da tecla que sera' usado para mover para baixo
     * @param d codigo da tecla que sera' usado para mover para esquerda
     * @param e codigo da tecla que sera' usado para mover para direita
     * @param a codigo da tecla que sera' usado para atirar
     */
    public Jogador(JogoTimer pai, String imagemNome, String bombaNome,
            int mx, int my, int c, int b, int d, int e, int a) {
        super(pai, imagemNome, mx, my);
        this.icone = this.carregarImagem(this.imagemNome);
        this.cima = c;
        this.baixo = b;
        this.direita = d;
        this.esquerda = e;
        this.atirar = a;
        this.qntDebombas = new ArrayList<Bomba>();
    }

    /**
     * Desenha a imagem no contexto gráfico do Pai
     * @param g 
     */
    @Override
    public void desenhar(Graphics2D g) {
        
        // percorre o vetor de bombas e desenha
        for (Bomba bomba1 : qntDebombas) {
            bomba1.desenhar(g);
        }

        // true mostra jogador, false nao mostra
        if (mostraJogador) { // serve para fazer o jogador piscar quando for atingido pela explosao
            g.drawImage(this.icone, this.posX, this.posY, this.jogo);
        }
    }

    /**
     * Atualiza coordenadas, posicao na matriz e a bomba
     */
    @Override
    // tempo de atualicao esta em 125ms
    public void atualizar() { // efetua todas as acoes, aqui dentro, a cada tempo de atualizacao
        int destinoLin = matLinha; // linha destino apos o movimento
        int destinoCol = matColuna; // coluna destino apos o movimento
        int destinoX = posX; // coordenada X destino na area de desenho
        int destinoY = posY; // coordenada Y destino na area de desenho

        if (jogo.framePrincipal.teclado.teclaApertada(this.cima)) {
            destinoY = posY - this.velocidade;
            destinoLin--;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.baixo)) {
            destinoY = posY + this.velocidade;
            destinoLin++;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.esquerda)) {
            destinoX = posX - this.velocidade;
            destinoCol--;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.direita)) {
            destinoX = posX + this.velocidade;
            destinoCol++;
        }

        //verifica se o destino linha ou coluna esta livre para o jogador poder se movimentar
        if (jogo.estaLivre(destinoLin, destinoCol)) {
            matLinha = destinoLin;
            matColuna = destinoCol;
            posX = destinoX;
            posY = destinoY;
            switch (jogo.cenario[destinoLin][destinoCol]) {
                case 5: // se a casa tem skill do tipo 5 (skill fogo), esta casa fica livre e alcanceBomba eh incrementado
                    this.jogo.cenario[destinoLin][destinoCol] = 2; // liberando casa com skill
                    this.alcanceBomba++;
                    break;
                case 6: // skill bomba
                    this.jogo.cenario[destinoLin][destinoCol] = 2;
                    this.bombasSimultaneas++;
                    break;
                case 7:
                    this.jogo.cenario[destinoLin][destinoCol] = 2;
                    tipoDeVeneno = this.jogo.varAleatoria.nextInt(2) + 1;
                    // decide se diminui bomba ou chama
                    if (tipoDeVeneno == 1) {
                        if (this.jogo.varAleatoria.nextInt(2) + 1 == 1) {
                            this.alcanceBomba = 1;
                        } else {
                            this.bombasSimultaneas = 1;
                        }
                    }
                    if (tipoDeVeneno == 2) {
                        this.inverterTeclasMovimentacao();
                    }

            }
        }
        
        //metodo para bomba
        if (jogo.framePrincipal.teclado.teclaApertadaPrimeiraVez(this.atirar)) {
            if (this.bombasSimultaneas > 0) {
                jogo.cenario[matLinha][matColuna] = 4; // esta casa fica ocupada com a bomba colocada pelo jogador
                this.bomba = new Bomba(this.jogo, "bomba.png", this.matColuna, this.matLinha, this.alcanceBomba); // cria uma bomba
                qntDebombas.add(bomba); // adciona a bomba ao vetor de bombas
                this.bombasSimultaneas--;
            }
        }

        // decrementa tempoAtingido de 2 em 2 e alterna o valor de mostraJogador entre true e false
        if (this.tempoAtingido > 0) { // tempoAtingido ganha 20 la no jogoTimer
            this.tempoAtingido -= 2; // decrementa de 2 em 2
            mostraJogador = !mostraJogador; // essa linha alterna entre true e false para fazer piscar o jogodor quando esse for atingido
        }
        
        // funcao para inverter teclas do jogador temporariamente
        if (this.tempoDeInversaoDeTecla > 0) {
            this.tempoDeInversaoDeTecla -= 2; 
            if (tempoDeInversaoDeTecla == 0) { 
                inverterTeclasMovimentacao(); // serve para desinverter teclas quando acabar o tempo do veneno (tempoDeInversaoDeTecla)
            }
        }
        
        for (Bomba bomba1 : qntDebombas) {
            bomba1.atualizar();
        }

        //verifica a quantidade de bombas que estao na array
        for (int i = 0; i < qntDebombas.size(); i++) {
            Bomba bombaVerifica = qntDebombas.get(i);//pegua a bomba e guarda em "i" para verificacao no if de baixo
            if (bombaVerifica.tempoDeExplosao <= 0) {
                qntDebombas.remove(i);//se a bomba da posicao "i" ja explodiu, entao ele eh excluida
                bombasSimultaneas++;//o jogador pode colocar mais uma bomba pois uma das bombas jah explodiu
            }
        }

        /*************************************************/
    }
    
    // metodo de inversao de teclas
    public void inverterTeclasMovimentacao() {
        int guardaCima = this.cima;
        int guardaBaixo = this.baixo;
        int guardaEsquerda = this.esquerda;
        int guardaDireita = this.direita;
        
        // funcao onde as teclas serao invertidas
        if (tempoDeInversaoDeTecla == 0) {
            this.cima = guardaBaixo;
            this.baixo = guardaCima;
            this.direita = guardaEsquerda;
            this.esquerda = guardaDireita;
            this.inverteuTeclas = !this.inverteuTeclas;
        }
        
        // funcao para acrescentar o tempo em que as teclas ficarao invertidas
        if (this.inverteuTeclas) {
            tempoDeInversaoDeTecla = 100; // 
        }
    }
}
