/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 * 
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import com.sun.media.sound.MidiUtils.TempoCache;
import com.sun.org.apache.xml.internal.utils.StopParseException;
import principal.JavaBomber;
import java.awt.event.ActionEvent;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import javax.swing.ImageIcon;
import javax.swing.Timer;
import java.util.Random;

/**
 * Essa clase contém a lógica principal do Jogo. A atualização da tela é feita
 * através de o Timer do pacote Swing. Por este motivo, essa classe tem que
 * implementar a Interface ActionListener
 * Essa classe herda de Canvas, pois a classe Canvas é ideal para fazer desenhos
 * e permite usufrir o bufferStrategy para melhorar a taxa de atualização da tela
 * 
 * @author Emerson Ribeiro de Mello
 */
public class JogoTimer extends Canvas implements ActionListener {

    public JavaBomber framePrincipal; // frame principal do jogo
    protected Jogador jog1; // Jogador1
    protected Jogador jog2; // Jogador2
    private Timer timer; // Timer responsável pela atualização do jogo
    public int larguraBloco; // Dimensao dos blocos, fundos. É 40
    public int colunas; // total de colunas na matriz do jogo
    public int linhas; // total de linhas na matriz do jogo
    private Image imgcenarios[]; // contém as imagens de fundo do cenário
    private int taxaDeAtualizacao; // a cada N milisegundos a tela é redesenhada
    protected Random varAleatoria = new Random(); // variavel para calculo de numeros aleatorios
    protected int vida = 3; // quantidades de vidas inicialmente de cada jogador
    protected boolean mostraRound = true; // mostra round
    protected int tempoMostraRound = 38; // tempo para mostrar round
    // 0 = ferro
    // 1 = tijolo
    // 2 = vazio
    // 3 = tijolo guardando skill
    // 4 = skill fogo
    // 5 = skill bomba
    // 6 = bomba que o jogador botou
    // 7 = veneno (perde itens ou inverte botoes de movimentacao)
    // 8 = 
    // 9 = 
    // ...
    public short[][] cenario = { //11x17 (linha,coluna)
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    };
    
    // cria cenarios aleatorios
    public void cenarioRandom(){
        int linha = 0;
        int coluna = 0;
        int casasVazias = 0;
        casasVazias = varAleatoria.nextInt(5) + 97; // sao ao todo 101 casas para colocar tijolos
        
        // esvazia todas as casas
        for (linha = 1; linha < 10; linha++) { // existem 10 linhas no cenario
            for (coluna = 1; coluna < 15; coluna++) { // existem 15 colunas no cenario
                if(cenario[linha][coluna] == 0){ // se for ferro deixa ferro senao esvazia
                    cenario[linha][coluna] = 0;
                }else{
                    cenario[linha][coluna] = 2;
                }
            }
        }
        
        // coloca tijolos aleatorios em todo o cenario
        for (int i = 0; i < casasVazias; i++) {
            
            linha = varAleatoria.nextInt(10) + 1; // escolhe aleatoriamente numeros para linha e coluna
            coluna = varAleatoria.nextInt(15) + 1;
            
            if(cenario[linha][coluna] == 0){ // verifica se a opcao sorteada eh ferro
                cenario[linha][coluna] = 0;
                i--;
            }else{
                cenario[linha][coluna] = 1;
            }
        }
        
        // retira tijolos do ponto de respawn
        cenario[1][1] = 2; 
        cenario[1][2] = 2;
        cenario[2][1] = 2;
        cenario[9][14] = 2;
        cenario[9][15] = 2;
        cenario[8][15] = 2;
        
        // coloca skills aleatoriamente embaixo dos tijolos
        for (int i = 0; i < 10; i++) { // sao espalhadas ao todo 10 itens pelo cenario (considerando skills e venenos)
            
            linha = varAleatoria.nextInt(10) + 1;
            coluna = varAleatoria.nextInt(15) + 1;
            
            if(cenario[linha][coluna] == 1){ // soh coloca a skill na posicao sorteada se esta for um tijolo
                cenario[linha][coluna] = 3;
            }else{
                i--;
            }
        }
    }

    public JogoTimer(JavaBomber p) {
        super();
        this.framePrincipal = p;
        this.setBackground(Color.GRAY);

        this.setIgnoreRepaint(true); // para melhorar a velocidade do redesenho
        this.larguraBloco = 40;
        this.colunas = 17;
        this.linhas = 11;
        this.taxaDeAtualizacao = 120;
        this.imgcenarios = new Image[11];
        // acrescenta as imagens que serao usadas no jogo a um vetor
        this.imgcenarios[0] = (new ImageIcon(this.getClass().getResource("/imagens/ferro.png"))).getImage();
        this.imgcenarios[1] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();
        this.imgcenarios[3] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();
        this.imgcenarios[5] = (new ImageIcon(this.getClass().getResource("/imagens/itemfogo.png"))).getImage();
        this.imgcenarios[6] = (new ImageIcon(this.getClass().getResource("/imagens/itembomba.png"))).getImage();
        this.imgcenarios[7] = (new ImageIcon(this.getClass().getResource("/imagens/veneno.png"))).getImage();
        this.imgcenarios[8] = (new ImageIcon(this.getClass().getResource("/imagens/round1.png"))).getImage();
        this.imgcenarios[9] = (new ImageIcon(this.getClass().getResource("/imagens/round2.png"))).getImage();
        this.imgcenarios[10] = (new ImageIcon(this.getClass().getResource("/imagens/empate.png"))).getImage();
    }

    /**
     * Método invocado para iniciar a partida
     */
    public void comecaOJogo() {
        // criando o Timer com taxa de atualização de N milisegundos
        this.timer = new Timer(taxaDeAtualizacao, this);
        // criando 2 buffer para acelerar a taxa de desenho
        createBufferStrategy(2);

        // KeyEvent possui codigos para as teclas. Por exemplo, para obter
        // o codigo da tecla 'seta para cima' basta invocar KeyEvent.VK_UP
        // Veja o uso do KeyEvent na criacao dos jogadores abaixo
        
        // Cria cenario randomico
        cenarioRandom();
        
        // criando Jogador 1
        this.jog1 = new Jogador(this, "azul.png", "bomba.png", 1, 1, 
                KeyEvent.VK_UP, KeyEvent.VK_DOWN, KeyEvent.VK_RIGHT, KeyEvent.VK_LEFT, KeyEvent.VK_SPACE);

        // criando Jogador 2
        this.jog2 = new Jogador(this, "vermelho.png", "bomba.png", 15, 9,
                KeyEvent.VK_W, KeyEvent.VK_S, KeyEvent.VK_D, KeyEvent.VK_A, KeyEvent.VK_1);


        // Disparando o timer para começar a atualização do Loop principal do Jogo
        // Todo o conteúdo do método actionPerformed dessa classe será executado
        // a cada 60 milisegundos.
        this.timer.start();
    }
    
    // pausa jogo
    public void pausaOJogo(){
        this.timer.stop(); // para o jogo do jeito que esta
    }
    
    // recomeca jogo
    public void recomecaOJogo(){
        this.timer.start(); // reinicia o jogo do pause
    }

    /**
     * Principal METODO do JOGO, pois e' aqui que fica a logica que fara':
     * - processamento da logica do jogo (morreu, disparou, etc..)
     * - atualizar as coordenadas dos elementos na tela para o proximo redesenho
     * - redesenhar os elementos na tela
     * 
     * Este método é invocado a cada N milisegundos pelo Timer
     * @param e 
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        this.framePrincipal.teclado.poll(); // processa os eventos no teclado
        this.processarLogica(); // atualiza as coordenadas dos elementos
        this.renderizar(); // desenha os elementos no buffer de desenho
        this.desenhar(); // faz a troca do buffer de desenho pelo buffer de apresentacao


        
    }

    /**
     * Atualiza as coordenadas/estados de todos os elementos presentes no jogo
     * No caso, somente 2 jogadores, mas aqui pode-se colocar qualquer outro
     * elemento que venha a ser incluido no jogo
     */
    
    // metodo que verifica se o jogador foi atingido pela explosao
    public void atingiuJogador(int linha, int coluna){
        // verifica se a explosao atingiu o jogador1 e se o tempoAtingido eh zero, para garantir que uma explosao em seguida nao tire duas vidas quase que simultaneamente
        if ((this.jog1.matLinha == linha && this.jog1.matColuna == coluna) && (this.jog1.tempoAtingido == 0)){
            this.jog1.vida--; // decrementa vida
            this.jog1.tempoAtingido = 20; // esta variavel eh decrementada gerando um tempo de recuperacao para o jogador atingido
            switch (this.jog1.vida) {
                case 0: // nesse caso jog1.vida esta em 0
                    this.framePrincipal.jLIconeJ1.setEnabled(false); // apaga icone de vida do jogador1
                    break;
                case 1:
                    this.framePrincipal.jLIconeJ5.setEnabled(false);
                    break;
                case 2:
                    this.framePrincipal.jLIconeJ6.setEnabled(false);
                    break;
                default:
                    break;
            }
        }
        if ((this.jog2.matLinha == linha && this.jog2.matColuna == coluna) && (this.jog2.tempoAtingido == 0)){
            this.jog2.vida--;
            this.jog2.tempoAtingido = 20;
            switch (this.jog2.vida) {
                case 0:
                    this.framePrincipal.jLIconeJ2.setEnabled(false);
                    break;
                case 1:
                    this.framePrincipal.jLIconeJ3.setEnabled(false);
                    break;
                case 2:
                    this.framePrincipal.jLIconeJ4.setEnabled(false);
                    break;
                default:
                    break;
            }
        }
        
    }
    
    public void processarLogica() {
        //Chama o atualizar dos elementos
        if (this.jog1 != null) {
            this.jog1.atualizar();
        }
        if (this.jog2 != null) {
            this.jog2.atualizar();
        }
    }

    /**
     * Renderiza no buffer secundário todos os elementos já com suas coordenadas/estados
     * atualizados.
     */
    public void renderizar() {
        Graphics g = getBufferStrategy().getDrawGraphics();
        Graphics2D g2 = (Graphics2D) g;
        //Limpando a tela

        // Gerando um gradiente que ficara' como cor de fundo
        //GradientPaint gp = new GradientPaint(25, 25, Color.white,50, 50, Color.RED, true);
        //g2.setPaint(gp);

        // ou pintando com uma cor definida por RGB
        Color c = new Color(255, 210, 174);
        g2.setColor(c);
        g2.fillRect(0, 0, getWidth(), getHeight());

        // desenhar cenario de fundo
        this.renderizarCenario(g2);

        // desenhando os elementos nao nulos
        if (this.jog1 != null) {
            this.jog1.desenhar(g2);
        }
        if (this.jog2 != null) {
            this.jog2.desenhar(g2);
        }
        
        // mostra round piscando na tela
        if (this.tempoMostraRound > 0){
            
            this.tempoMostraRound -= 2;
            mostraRound = !mostraRound;
        }
        
        // mostra rounds na tela
        if (mostraRound){
            if(this.jog1.vida == 3 || this.jog2.vida == 3){
                g2.drawImage(this.imgcenarios[8], 180, 150, this.framePrincipal); // round 1 na tela
            }
        }
        
        //liberando os contextos gráficos
        g.dispose();
        g2.dispose();

    }

    /**
     * Efetuando a troca das areas de desenho e apresentacao. 
     * No metodo redenrizar os elemtos graficos foram desenhados em um buffer
     * secundario, agora iremos fazer com que esse buffer secundario torne-se
     * primario, ou seja, que apareca de fato na tela
     */
    public void desenhar() {
        if (!getBufferStrategy().contentsLost()) {
            getBufferStrategy().show();
        }
    }
    
    

    /**
     * Fim de jogo/rodada
     */
    public void finalizar() {
        this.jog1 = null;
        this.jog2 = null;
        this.timer.stop();
    }

    /**
     * Desenha o cenário com as paredes de tijolos e de ferro
     * @param g contexto gráfico
     */
    public void renderizarCenario(Graphics2D g) {
        for (int i = 0; i < 11; i++) { // linhas
            for (int j = 0; j < 17; j++) { // colunas
                if (this.cenario[i][j] != 2) { // se não for vazio, desenhe
                    g.drawImage(this.imgcenarios[this.cenario[i][j]], this.larguraBloco * j, this.larguraBloco * i, this.framePrincipal);
                }
            }
        }
    }

    /**
     * Verifica se na posicao lin,col da matriz tem um espaço vazio
     * @param lin numero da linha
     * @param col numero da coluna
     * @return true se estiver vazio, false caso contrario
     */
    public boolean estaLivre(int lin, int col) {
        if(cenario[lin][col] == 2){
            return true; // se for == 2, entao ira' retornar true, false caso contrario
        } else if((cenario[lin][col] == 5) || (cenario[lin][col] == 6) || (cenario[lin][col] == 7)){ // os estados 5 (skill fogo), 6 (skill bomba) e 7 (veneno) sao consideradas casas livres
            return true; 
        }
        return false; // no caso de ser ferro, bomba ou tijolo
    }

    /**
     * Indica em que posicao na tela encontrase o inicio dessa o retangulo que 
     * compreende uma celula
     * @param celula posicao na matriz que deseja-se descobrir
     * @return Coordenada na tela
     */
    public int celulaParaPixel(int celula) {
        return larguraBloco * celula;

    }

    /**
     * Indica qual celula o pixel esta' contido na matriz
     * @param pixel
     * @return 
     */
    public int pixelParaCelula(int pixel) {
        return pixel / larguraBloco;
    }
}
